#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, locale, uuid
import xml.dom.minidom as xml


########################################################################
class effect:
	"""This class represents a media clip on the timeline."""

	#----------------------------------------------------------------------
	def __init__(self, service, paramaters=[]):
		"""Constructor"""
		
		# init variables for clip object
		self.service = service					# the name of the mlt service (i.e. frei0r.water, chroma, sox, etc...)
		self.paramaters = paramaters			# example:  "key" : "123123123",   "variance" : "0.15" (dictionary of settings)
		self.audio_effect = ""
		self.unique_id = str(uuid.uuid1())
		
		if "sox" in service:
			self.audio_effect = service[4:]


	def isnumeric(self, value):
		return str(value).replace(".", "").replace("-", "").replace(",", "").isdigit()


	def GenerateXML(self, dom, parent_node, in_frame=0.0, out_frame=0.0, track=1):
		print "GenerateXML for an Effect"
		
		service = self.service
		audio_effect = ""
		
		if "sox" in service:
			audio_effect = service[4:]
			service = "sox"
		
		# create effect node
		filter_root = dom.createElement("filter")
		filter_root.setAttribute("mlt_service", service)
		filter_root.setAttribute("in", str(in_frame))
		filter_root.setAttribute("out", str(out_frame))
		filter_root.setAttribute("track", str(track))
		
		if not audio_effect:
			# VIDEO EFFECT
			# loop through each parameter of this effect, and add a node to the effect XML
			for item in self.paramaters:
				# get key and value
				k = item.items()[0][0]
				v = item.items()[0][1]
				
				# format value for locale (if it's a number)
				if self.isnumeric(v):
					v = locale.str(float(v))					
				
				# add property node
				property_node = dom.createElement("property")
				property_node.setAttribute("name", k)
				# add value
				text = dom.createTextNode(v)
				property_node.appendChild(text)
				# append to the filter node
				filter_root.appendChild(property_node)
		else:
			# AUDIO EFFECT
				# add property node
				property_node = dom.createElement("property")
				property_node.setAttribute("name", "effect1")
				
				# concat all sox params into a single string
				sox_value = ""
				for item in self.paramaters:
					# get key and value
					k = item.items()[0][0]
					v = item.items()[0][1]
					
					# format value for locale (if it's a number)
					if self.isnumeric(v):
						v = locale.str(float(v))	
					
					sox_value = sox_value + " " + v
					
				# add value
				text = dom.createTextNode("%s %s%s" % (audio_effect, audio_effect, sox_value))
				property_node.appendChild(text)
				# append to the filter node
				filter_root.appendChild(property_node)
		
		# add effect node to parent
		parent_node.appendChild(filter_root)
		
		
	#----------------------------------------------------------------------
	def __setstate__(self, state):
		""" This method is called when an OpenShot project file is un-pickled (i.e. opened).  It can
		    be used to update the structure of old clip classes, to make old project files compatable with
		    newer versions of OpenShot. """
	
		# Check for missing attributes (which means it's an old project format)
		if 'unique_id' not in state:
			state['unique_id'] = str(uuid.uuid1())

		# update the state object with new schema changes
		self.__dict__.update(state)
		
		
		
class effect_metadata:
	""" Class to hold meta data for an effect. """
	#----------------------------------------------------------------------
	def __init__(self):
		"""Constructor"""
		self.title = ""
		self.description = ""
		self.icon = ""
		self.category = ""
		self.service = ""
		self.audio_effect = ""
		self.params = []
		
	def get_default_params(self):
		output = []
		
		# loop through params
		for param in self.params:
			# get name and default value
			dict_param = {}
			dict_param[param.name] = param.default
			output.append(dict_param)
			
		# return default params
		return output
		
		
class effect_param_metadata:
	""" Class to hold meta data for an effect. """
	#----------------------------------------------------------------------
	def __init__(self):
		"""Constructor"""
		self.title = ""
		self.description = ""
		self.name = ""
		self.type = ""
		self.min = 0.0
		self.max = 0.0
		self.default = 0.0
		self.values = {}


	
def compare_effect(MyEffect1, MyEffect2):
	if MyEffect1.title > MyEffect2.title:
		return 1
	elif MyEffect1.title == MyEffect2.title:
		return 0
	else:
		return -1

	

def get_effects(project=None, directory=None):
	""" Get a list of effect_metatdata objects. """ 
	EFFECTS_DIR = ""
	
	if directory:
		EFFECTS_DIR = directory
	elif project:
		EFFECTS_DIR = project.EFFECTS_DIR
	else:
		EFFECTS_DIR = "/home/jonathan/openshot/openshot/effects"
		
	#parse the xml files and get targets that match the project type
	effects = []
	
	for file in os.listdir(EFFECTS_DIR):
		if os.path.isfile(os.path.join(EFFECTS_DIR, file)) and ".xml" in file:
			# load xml effect file
			xmldoc = xml.parse(os.path.join(EFFECTS_DIR, file))
			
			# create effect_metadata class
			effect1 = effect_metadata()
			
			effect1.title = xmldoc.getElementsByTagName("title")[0].childNodes[0].data
			effect1.description = xmldoc.getElementsByTagName("description")[0].childNodes[0].data
			effect1.icon = xmldoc.getElementsByTagName("icon")[0].childNodes[0].data
			effect1.category = xmldoc.getElementsByTagName("category")[0].childNodes[0].data
			effect1.service = xmldoc.getElementsByTagName("service")[0].childNodes[0].data
			
			if "sox" in effect1.service:
				effect1.audio_effect = effect1.service[4:]
				effect1.service = "sox"
			
			params = xmldoc.getElementsByTagName("param")
			for param in params:
				# create effect_param_metadata object
				param1 = effect_param_metadata()
				
				if param.attributes["title"]:
					param1.title = param.attributes["title"].value
				
				if param.attributes["description"]:
					param1.description = param.attributes["description"].value
					
				if param.attributes["name"]:
					param1.name = param.attributes["name"].value
					
				if param.attributes["type"]:
					param1.type = param.attributes["type"].value
					
				if param.getElementsByTagName("min"):
					param1.min = param.getElementsByTagName("min")[0].childNodes[0].data
					
				if param.getElementsByTagName("max"):
					param1.max = param.getElementsByTagName("max")[0].childNodes[0].data
					
				if param.getElementsByTagName("default"):
					param1.default = param.getElementsByTagName("default")[0].childNodes[0].data
					
				values = param.getElementsByTagName("value")
				for value in values:
					# create effect_param_metadata object
					name = ""
					num = ""
					
					if value.attributes["name"]:
						name = value.attributes["name"].value
						
					if value.attributes["num"]:
						num = value.attributes["num"].value
						
					# add to parameter
					param1.values[name] = num
					
				# Append param to effect
				effect1.params.append(param1)
					
				
			# add effect to list
			effects.append(effect1)
			
	# get a list of all clips on this track
	effects.sort(compare_effect)

	# return effect list
	return effects



def main():

	# Get list of all effects
	effect1 = get_effects()[0]
	print effect1
	print effect1.get_default_params()
	

if __name__ == "__main__":
	main()
	
	
	
	
	


#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, threading, time, uuid


def check_directory_present(path, permissions = 0750):
	"""Confirm a directory exists and has the requested permissions.
	Returns False if the directory cannot be re-created."""

	ret = False

	if not os.path.exists(path):
		try:
			os.mkdir(path, permissions)
			ret = True

		except OSError:
			# TODO: Friendlier, GUI-based, non-fatal, exception handling needed
			sys.exit("Fatal: cannot create %s" % path)
	else:
		# TODO: check the permissions match and if not attempt to correct them
		ret = True

	return ret

def check_folders_exist(path):
	# ensure the user's ~/.openshot directory exists
	if not check_directory_present(path):
		# TODO: Friendlier, GUI-based, non-fatal, exception handling needed
		sys.exit("Fatal: cannot create %s" % path)

	# check that all sub-directories exist
	if not check_directory_present(os.path.join(path, "queue")):
		# TODO: Friendlier, GUI-based, non-fatal, exception handling needed
		sys.exit("Fatal: cannot create %s" % os.path.join(path, "queue"))

	if not check_directory_present(os.path.join(path, "thumbnail")):
		# TODO: Friendlier, GUI-based, non-fatal, exception handling needed
		sys.exit("Fatal: cannot create %s" % os.path.join(path, "thumbnail"))
		
	if not check_directory_present(os.path.join(path, "user_profiles")):
		# TODO: Friendlier, GUI-based, non-fatal, exception handling needed
		sys.exit("Fatal: cannot create %s" % os.path.join(path, "user_profiles"))
		
	if not check_directory_present(os.path.join(path, "blender")):
		# TODO: Friendlier, GUI-based, non-fatal, exception handling needed
		sys.exit("Fatal: cannot create %s" % os.path.join(path, "blender"))


def check_pid(path):
	""" Check and see if this is the only instance of OpenShot. If not, then kill this instance. """

	# ensure folders exist
	check_folders_exist(path)

	# get path to lock file
	pidPath = os.path.join(path, "pid.lock")
	
	if os.path.exists(pidPath):
		pid = 0
		is_running = False
		number_of_files = 0
		
		# get pid and check if pid is running (this doesn't kill it)
		try:
			# pid file exists
			pid=int(open(pidPath, 'r').read().strip())
			
			# does process exist?
			os.kill(pid, 0)
			
			# OpenShot is already running, so...
			is_running = True
			
		except:
			print "Process no longer exists: %s.  Creating new pid lock file." % pid
			
			# not running anymore (maybe program crashed... and left this pid file)
			fp=open(pidPath, 'w')
			fp.write(str(os.getpid()))
			fp.close()
			
			# OpenShot is not currently running, so...
			is_running = False
		
		# OpenShot is alreay running, so kill this instance (ONLY KILL IF IT CONTAINS ARGS)
		# The user should be able to run as many instances of OpenShot as needed... but when ARGS
		# are passed, it should only allow 1 instance
		if len(sys.argv) > 1:

			# loop through the remaining args
			print "Adding files to the watch queue:"
			for arg in sys.argv[1:]:
				
				# ignore OSP project files
				if ".osp" not in arg:
					# print the path of the media file
					print arg
					
					# create a import queue file for the primary instance of OpenShot
					fp=open(os.path.join(path, "queue", str(uuid.uuid1())), 'w')
					fp.write(arg)
					fp.close()
					
					# increment counter
					number_of_files += 1
				
			# exit the program (if OpenShot is already running) and ARGV is passed in
			if is_running and number_of_files:
				sys.exit("Another instance of this program is already running")

	else:
		# pid file doesn't exist
		fp=open(pidPath, 'w')
		fp.write(str(os.getpid()))
		fp.close()



class queue_watcher ( threading.Thread ):
	""" This class polls the /queue/ folder, looking for files to import into OpenShot.  When it finds
	a text file, it should get the path out of the file, import the file, and then delete the file.  Only
	1 instance of OpenShot should be polling this folder. """
	
	def set_form(self, main_form):
		self.form = main_form

	def run ( self ):
		""" This is the main method on this thread.  This method should not return anything, or the 
		thread will no longer be active...  """
		
		import gobject

		self.path = self.form.project.USER_DIR
		self.queue_location = os.path.join(self.path, "queue")
		pidPath = os.path.join(self.path, "pid.lock")
		self.amAlive = True
		pid=int(open(pidPath, 'r').read().strip())
		
		# only allow this thread to run if this instance of OpenShot is the primary instance.
		# we can't have 2 instances both watching the /queue/ folder.
		if os.getpid() == pid:

			# this loop will continue as long as OpenShot is running
			while self.amAlive:
				needs_refresh = False

				# check for files in the /queue/ folder
				for filename in os.listdir(self.queue_location):
					# get full file path
					full_filename = os.path.join(self.queue_location, filename)
					
					# read the content of the file
					import_path = open(full_filename, 'r').read().strip()

					# IMPORT FILE
					if self.form.project.project_folder:
						gobject.idle_add(self.form.project.project_folder.AddFile, import_path)
					needs_refresh = True
	
			 		# delete import file
			 		os.remove(full_filename)
			 		
			 	# refresh project files list (if needed)
			 	if needs_refresh:
			 		if self.form:
			 			# change modified status
			 			gobject.idle_add(self.form.project.set_project_modified, True, False)
			 			# refresh form
			 			gobject.idle_add(self.form.refresh)

			 	# wait 2 seconds
				time.sleep(1) 
		
		else:
			print "Not the primary instance of OpenShot. Not starting queue watcher thread."



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import gtk, os, sys
from xdg.IconTheme import *

class EntryDialog(gtk.Dialog):

        def __init__(self, message="", default_text="", modal=True):
        
                
                gtk.Dialog.__init__(self)
                self.connect("destroy", self.quit)
                self.connect("delete_event", self.quit)
                if modal:
                        self.set_modal(True)
			
		self.set_title("Openshot")
		
		self.set_resizable(False)
                        
                if getIconPath("openshot"):
			self.set_icon_from_file(getIconPath("openshot"))
        
                box = gtk.VBox(spacing=10)
                box.set_border_width(10)
                self.vbox.pack_start(box)
                box.show()
                if message:
                        label = gtk.Label(message)
                box.pack_start(label)
        
                label.show()
                self.entry = gtk.Entry()
		#Enable the user to hit the enter key
		self.entry.connect("activate",self.ok_clicked)
                self.entry.set_text(default_text)
                box.pack_start(self.entry)
                self.entry.show()
                self.entry.grab_focus()
                
                cancel_button = gtk.Button(stock=gtk.STOCK_CANCEL)
                cancel_button.connect("clicked", self.quit)
                cancel_button.set_flags(gtk.CAN_DEFAULT)
                self.action_area.pack_start(cancel_button)
                cancel_button.show()
                
                ok_button = gtk.Button(stock=gtk.STOCK_OK)
                ok_button.connect("clicked", self.ok_clicked)
                ok_button.set_flags(gtk.CAN_DEFAULT)
                self.action_area.pack_start(ok_button)
                ok_button.show()
                ok_button.grab_default()
               
                self.ret = None
		
		self.run()
		
        
        def quit(self, w=None, event=None):
        
                self.hide()
                self.destroy()
        
                
        
        def ok_clicked(self, button):
        
                self.ret = self.entry.get_text()
                self.quit()
        
        
#call this to display an input box
def input_box(title="", message="", default_text="", modal=True):
        
        win = EntryDialog(message, default_text, modal=modal)
        
        return win.ret




#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas, TJ
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, locale
import gtk
import xml.dom.minidom as xml
from classes import profiles, files, thumbnail, open_project, save_project, state_project, restore_state, sequences, video, theme

# init the foreign language
from language import Language_Init

########################################################################
class project():
	"""This is the main project class that contains all
	the details of a project, such as name, folder, timeline
	information, sequences, media files, etc..."""

	#----------------------------------------------------------------------
	def __init__(self, init_threads=True):
		"""Constructor"""
		
		# debug message/function control
		self.DEBUG = True
		
		# define common directories containing resources
		# get the base directory of the openshot installation for all future relative references
		# Note: don't rely on __file__ to be an absolute path. E.g., in the debugger (pdb) it will be
		# a relative path, so use os.path.abspath()
		self.BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
		self.UI_DIR = os.path.join(self.BASE_DIR, "openshot", "windows", "ui")
		self.IMAGE_DIR = os.path.join(self.BASE_DIR, "openshot", "images")
		self.LOCALE_DIR = os.path.join(self.BASE_DIR, "openshot", "locale")
		self.PROFILES_DIR = os.path.join(self.BASE_DIR, "openshot", "profiles")
		self.TRANSITIONS_DIR = os.path.join(self.BASE_DIR, "openshot", "transitions")
		self.BLENDER_DIR = os.path.join(self.BASE_DIR, "openshot", "blender")
		self.EXPORT_PRESETS_DIR = os.path.join(self.BASE_DIR, "openshot", "export_presets")
		self.EFFECTS_DIR = os.path.join(self.BASE_DIR, "openshot", "effects")
		# location for per-session, per-user, files to be written/read to
		self.DESKTOP = os.path.join(os.path.expanduser("~"), "Desktop")
		self.USER_DIR = os.path.join(os.path.expanduser("~"), ".openshot")
		self.THEMES_DIR = os.path.join(self.BASE_DIR, "openshot", "themes")
		self.USER_PROFILES_DIR = os.path.join(self.USER_DIR, "user_profiles")
		self.USER_TRANSITIONS_DIR = os.path.join(self.USER_DIR, "user_transitions")
		

		# only run the following code if we are really using 
		# this project file... 
		if init_threads:

			# Add language support
			translator = Language_Init.Translator(self)
			_ = translator.lang.gettext
			
			# init the variables for the project
			from windows import preferences
			self.name = _("Default Project")
			self.folder = self.USER_DIR
			self.project_type = preferences.Settings.general["default_profile"]
			self.canvas = None
			self.is_modified = False
			self.refresh_xml = True
			self.mlt_profile = None
			
			# set theme
			self.set_theme(preferences.Settings.general["default_theme"])
			
			# reference to the main GTK form
			self.form = None
			
			# init the file / folder list (used to populate the tree)
			self.project_folder = files.OpenShotFolder(self)
	
			# ini the sequences collection
			self.sequences = [sequences.sequence(_("Default Sequence 1"), self)]		
	
			# init the tab collection
			self.tabs = [self.sequences[0]]	  # holds a refernce to the sequences, and the order of the tabs
	
			# clear temp folder
			self.clear_temp_folder()
			
			# create thumbnailer object
			self.thumbnailer = thumbnail.thumbnailer()
			self.thumbnailer.set_project(self)
			self.thumbnailer.start()
			
	def set_theme(self, folder_name):
		""" Set the current theme and theme settings """ 

		# Set the theme, and load the theme settings
		self.theme = folder_name
		self.theme_settings = theme.theme(folder_name=self.theme, project=self)
		
		# check for empty theme settings (and use blue_glass instead if needed)
		if not self.theme_settings.settings:
			self.theme = "blue_glass"
			self.theme_settings = theme.theme(folder_name=self.theme, project=self)
		
		
	def fps(self):
		# get the profile object
		if self.mlt_profile == None:
			self.mlt_profile = profiles.mlt_profiles(self).get_profile(self.project_type)

		# return the frames per second
		return self.mlt_profile.fps()
	
	

	def clear_temp_folder(self):
		"""This method deletes all files in the /openshot/temp folder."""
		path = os.path.join(self.USER_DIR)
		
		# get pid from lock file
		pidPath = os.path.join(path, "pid.lock")
		pid=int(open(pidPath, 'r').read().strip())
		
		# list of folders that should not be deleted
		safe_folders = ["blender", "queue", "user_profiles", "user_transitions"]
		
		# loop through all folders in the USER_DIR
		for child_path in os.listdir(path):
			if os.path.isdir(os.path.join(path, child_path)):
				if child_path not in safe_folders:
					# clear all files / folders recursively in the thumbnail folder
					if os.getpid() == pid:

						# only clear this folder for the primary instance of OpenShot
						self.remove_files(os.path.join(path, child_path))
						
						# remove folder
						os.removedirs(os.path.join(path, child_path))
		
		# thumbnail path
		thumbnail_path = os.path.join(path, "thumbnail")
		
		# create thumbnail folder (if it doesn't exist)
		if os.path.exists(thumbnail_path) == False:
			# create new thumbnail folder
			os.mkdir(thumbnail_path)


	def remove_files(self, path):

		# verify this folder exists
		if os.path.exists(path):
			
			# loop through all files in this folder
			for child_path in os.listdir(path):
				# get full child path
				child_path_full = os.path.join(path, child_path)
				
				if os.path.isdir(child_path_full) == True:
					# remove items in this folder
					self.remove_files(child_path_full)
					
					# remove folder
					os.removedirs(child_path_full)
				else:
					# remove file
					os.remove(child_path_full)
			
			
	#----------------------------------------------------------------------
	def __setstate__(self, state):
		""" This method is called when an OpenShot project file is un-pickled (i.e. opened).  It can
		    be used to update the structure of the old project class, to make old project files compatable with
		    newer versions of OpenShot. """
	
		# Check for missing DEBUG attribute (which means it's an old project format)
		#if 'DEBUG' not in state:
		# create empty new project class
		empty_project = project(init_threads=False)
		
		state['DEBUG'] = empty_project.DEBUG
		state['BASE_DIR'] = empty_project.BASE_DIR
		state['UI_DIR'] = empty_project.UI_DIR
		state['IMAGE_DIR'] = empty_project.IMAGE_DIR
		state['LOCALE_DIR'] = empty_project.LOCALE_DIR
		state['PROFILES_DIR'] = empty_project.PROFILES_DIR
		state['TRANSITIONS_DIR'] = empty_project.TRANSITIONS_DIR
		state['BLENDER_DIR'] = empty_project.BLENDER_DIR
		state['EXPORT_PRESETS_DIR'] = empty_project.EXPORT_PRESETS_DIR
		state['EFFECTS_DIR'] = empty_project.EFFECTS_DIR
		state['USER_DIR'] = empty_project.USER_DIR
		state['DESKTOP'] = empty_project.DESKTOP
		state['THEMES_DIR'] = empty_project.THEMES_DIR
		state['USER_PROFILES_DIR'] = empty_project.USER_PROFILES_DIR
		state['USER_TRANSITIONS_DIR'] = empty_project.USER_TRANSITIONS_DIR
		state['refresh_xml'] = True
		state['mlt_profile'] = None

		empty_project = None

		# update the state object with new schema changes
		self.__dict__.update(state)


	#----------------------------------------------------------------------
	def Render(self):
		"""This method recursively renders all the tracks and clips on the timeline"""
		
		# Render the timeline
		self.sequences[0].Render()
		
		# Render Play Head (and position line)
		self.sequences[0].RenderPlayHead()
		
		
	def GenerateXML(self, file_name):
		"""This method creates the MLT XML used by OpenShot"""
		
		# get locale info
		lc, encoding = locale.getdefaultlocale()

		# Create the XML document
		dom = xml.Document()
		dom.encoding = encoding

		# Add the root element
		westley_root = dom.createElement("mlt")
		dom.appendChild(westley_root)
		tractor1 = dom.createElement("tractor")
		tractor1.setAttribute("id", "tractor0")
		westley_root.appendChild(tractor1)
		
		# Add all the other timeline objects (such as sequences, clips, filters, and transitions)
		self.sequences[0].GenerateXML(dom, tractor1)
		
		# Save the XML dom
		f = open(file_name, "w")
		dom.writexml(f, encoding=encoding)
		f.close()
		
		# reset project as NOT modified
		self.refresh_xml = False


	#----------------------------------------------------------------------
	def RefreshXML(self):
		""" Generate a new MLT XML file (if needed).  This only creates a
		new XML file if the timeline has changed. """
		
		# has the project timeline been modified (i.e. new clips, re-arranged clips, etc...)
		if self.refresh_xml:
			
			# update cursor to "wait"
			self.form.timelinewindowRight.window.set_cursor(gtk.gdk.Cursor(150))
			self.form.timelinewindowRight.window.set_cursor(gtk.gdk.Cursor(150))
			
			# generate a new MLT XML file
			self.GenerateXML(os.path.join(self.USER_DIR, "sequence.mlt"))

			# ****************************
			# re-load the xml
			if self.form.MyVideo:
				# store current frame position
				prev_position = self.form.MyVideo.position()

				self.form.MyVideo.set_project(self, self.form, os.path.join(self.USER_DIR, "sequence.mlt"), mode="preview")
				self.form.MyVideo.load_xml()

				# restore position
				self.form.MyVideo.seek(prev_position)

			else:
				# create the video thread
				# Force SDL to write on our drawing area
				os.putenv('SDL_WINDOWID', str(self.form.videoscreen.window.xid))
				
				# We need to flush the XLib event loop otherwise we can't
				# access the XWindow which set_mode() requires
				gtk.gdk.flush()

				# play the video in it's own thread
				self.form.MyVideo = video.player(self, self.form, os.path.join(self.USER_DIR, "sequence.mlt"), mode="preview")
				self.form.MyVideo.start()
			# ****************************
			
			# update cursor to normal
			self.form.timelinewindowRight.window.set_cursor(None)
			
		else:
			pass

	
	#----------------------------------------------------------------------
	def Save(self, file_path):
		"""Call the save method of this project, which will 
		persist the project to the file system."""
		
		# get preferences to see whether to save in binary or ascii form
		from windows import preferences
		self.file_type = preferences.Settings.general["project_file_type"]
		
		# call the save method
		save_project.save_project(self, file_path)
		
	#----------------------------------------------------------------------
	def Open(self, file_path):
		"""Call the open method, which will open an existing
		project file from the file system."""
			
		# call the open method
		open_project.open_project(self, file_path)
		self.set_project_modified(is_modified=True, refresh_xml=False, type=_("Opened project"))	

		
	def set_project_modified(self, is_modified=False, refresh_xml=False, type=None):
		"""Set the modified status and accordingly the save button sensitivity"""
		self.is_modified = is_modified
		self.refresh_xml = refresh_xml

		if is_modified == True:
			self.form.tlbSave.set_sensitive(True)
			### test undo : debug
			if type:
				self.form.save_project_state(type)
		else:
			self.form.tlbSave.set_sensitive(False)
			
	
	def State(self):
		state = state_project.save_state(self)
		return state
	state = property(State)
	
	
	def restore(self, state):
		
		# Restore State
		restore_state.restore_project_state(self, state)
		
	def translate(self, text):
		""" Translate any string to the current locale. """
		return self.form.translate(text)
				
		



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import gtk, goocanvas


########################################################################
class marker:
	"""This class represents a marker (i.e. a reference point) on the timeline."""

	#----------------------------------------------------------------------
	def __init__(self, marker_name, position, parent):
		"""Constructor"""
		self.name = marker_name
		self.position_on_track = float(position)
		self.parent = parent
		
	#----------------------------------------------------------------------
	def Render(self):
		
		# get the previous track from the parent sequence (if any)
		pixels_per_second = self.parent.get_pixels_per_second()
		y_top = 22
		
		# get a reference to the 2 main canvas objects & theme
		theme = self.parent.project.theme
		canvas_left = self.parent.project.form.TimelineCanvas_Left
		canvas_right = self.parent.project.form.TimelineCanvas_Right
		
		# Add an item to the goocanvas
		root_left = canvas_left.get_root_item ()
		root_right = canvas_right.get_root_item ()
		
		# load marker image
		imgMarker = gtk.image_new_from_file("%s/openshot/themes/%s/marker.png" % (self.parent.project.form.openshot_path, theme))
		imgMarker_Width = imgMarker.get_pixbuf().get_width()
		offset = float(imgMarker_Width) / float(2.0)
		
		# determine position
		x = pixels_per_second * self.position_on_track
		
		# Add Left Image to Group
		image1 = goocanvas.Image (parent = root_right,
								  pixbuf = imgMarker.get_pixbuf(),
								  x = x - offset,
								  y = y_top)
		
		# attach button click event to marker image
		image1.connect ("button_press_event", self.on_marker_press)
		
		
	def on_marker_press (self, item, target, event):
		""" This is the click signal for a marker. """
		
		if event.button == 3:
			# show the marker popup menu
			self.parent.project.form.mnuMarker1.showmnu(event, self)
 
		return True



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os
import gtk
from classes import messagebox, project, effect
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp

# init the foreign language
from language import Language_Init

class frmAddEffect(SimpleGtkBuilderApp):

	def __init__(self, path="AddEffect.ui", root="frmAddEffect", domain="OpenShot", parent=None, form=None, project=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext

		self.form = form
		self.project = project
		self.parent = parent
		self.frmAddEffect.show_all()
		
		# Init List of Effects
		effect_list = self.form.effect_list
		
		# Add effects to dropdown
		for my_effect in effect_list:
			
			# is frei0r effect library installed?
			if self.form.has_frei0r_installed == False and my_effect.service.startswith("frei0r"):
				# frei0r not installed, and this is a frei0r effect
				# skip to next item in loop
				continue
			
			# add item
			self.cboEffects.append_text(my_effect.title)
		
		
		
	def on_btnCancel_clicked(self, widget, *args):
		print "on_btnCancel_clicked"
		self.frmAddEffect.destroy()
		
		
	def on_frmAddEffect_destroy(self, widget, *args):
		print "on_frmAddEffect_destroy"
		
	def on_frmAddEffect_close(self, widget, *args):
		print "on_frmAddEffect_close"
		self.frmAddEffect.destroy()
		
	def on_frmAddEffect_destroy(self, widget, *args):
		print "on_frmAddEffect_destroy"
		
	def on_frmAddEffect_response(self, widget, *args):
		print "on_frmAddEffect_response"
		
	def on_btnOk_clicked(self, widget, *args):
		print "on_btnOk_clicked"

		# get Service name
		effect_title = self.cboEffects.get_active_text()
		
		# Add the effect
		if effect_title:
			# get real effect object
			real_effect = self.parent.OSTreeEffects.get_real_effect(title=effect_title)
			
			# add effect
			if real_effect.audio_effect:
				# audio effect
				self.parent.copy_of_clip.Add_Effect("%s:%s" % (real_effect.service, real_effect.audio_effect))
			else:
				# service
				self.parent.copy_of_clip.Add_Effect(real_effect.service)
			
			# update effect tree
			self.parent.update_effects_tree()
		
		# close window
		self.frmAddEffect.destroy()


		
		
			
def main():
	frmImportImageSequence1 = frmImportImageSequence()
	frmImportImageSequence1.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os
import gtk, pango
from classes import messagebox, project, timeline
from windows.SimpleGtkBuilderApp import SimpleGtkBuilderApp

# init the foreign language
from language import Language_Init


class frmFileproperties(SimpleGtkBuilderApp):

	def __init__(self, file, path="FileProperties.ui", root="frmFileProperties", domain="OpenShot", form=None, project=None, **kwargs):
		SimpleGtkBuilderApp.__init__(self, os.path.join(project.UI_DIR, path), root, domain, **kwargs)

		# Add language support
		_ = Language_Init.Translator(project).lang.gettext
		
		self.form = form
		self.project = project
		self.file = file
		
		#set the thumbnail - use the preview thumbnail
		#for video & image files
		pixbuf = file.get_thumbnail(112, 83)
		self.imgPreview.set_from_pixbuf(pixbuf)
		
		#set the file type	
		self.lblMimeType.set_label(file.file_type)
		
		#if the file name is too long to fit the space, add ellipses and a tooltip.
		self.lblLocation1.set_text(file.name)
		#self.lblLocation1.set_tooltip_text(file.name)
		#self.lblLocation1.set_ellipsize(pango.ELLIPSIZE_END)
		#format the file length
		milliseconds = file.length * 1000
		time = timeline.timeline().get_friendly_time(milliseconds)
	
		hours = time[2]
		mins = time[3]
		secs = time[4]
		milli = time[5]
	
		time_str =  "%02d:%02d:%02d" % (time[2], time[3], time[4])
		self.lblLengthValue.set_label(time_str)
		#project label
		self.txtLabel.set_text(file.label)
		
		if file.file_type.startswith("video"):
		#video specific properties
			self.lblRateValue.set_label(str(file.videorate[0]) + "/" + str(file.videorate[1]))
			self.lblSizeValue.set_label(str(file.width) + " x " + str(file.height))
			
		else:
			self.lblRate.set_sensitive(False)
			self.lblSize.set_sensitive(False)
		
		#show the form
		self.frmFileProperties.show_all()
		
	def on_btnClose_clicked(self, event, *args):
		self.frmFileProperties.destroy()
		
	def on_btnApply_clicked(self, event, *args):
		
		# update path of file
		self.file.name = self.lblLocation1.get_text()
		
		if self.txtLabel.get_text() == "":
			return

		self.project.project_folder.UpdateFileLabel(self.lblLocation1.get_text(), self.txtLabel.get_text(), 1)
		self.frmFileProperties.destroy()
		
def main():
	frm_file_properties = frmFileProperties()
	frm_file_properties.run()

if __name__ == "__main__":
	main()



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys
import gtk, goocanvas
import xml.dom.minidom as xml

from classes import clip, files, marker, timeline, track
# init the foreign language
from language import Language_Init

########################################################################
class sequence:
	"""A sequence contains tracks and clips that make up a scene (aka sequence).  Currently, Openshot
	only contains a single sequence, but soon it will have the ability to create many sequences."""

	#----------------------------------------------------------------------
	def __init__(self, seq_name, project):
		"""Constructor"""
		
		# Add language support
		translator = Language_Init.Translator(project)
		_ = translator.lang.gettext
				
		# init variables for sequence
		self.name = seq_name
		self.length = 600.0		   # length in seconds of sequence.  This controls how wide to render the tracks.
		self.project = project	  # reference to current project
		self.scale = 15.0		   # this represents how many seconds per tick mark
		self.tick_pixels = 100	  # number of pixels between each tick mark
		self.play_head_position = 0 # position of the play head in seconds
		
		# init the tracks on the sequence
		self.tracks = []
		self.tracks.append(track.track(_("Track %s") % 2, self))
		self.tracks.append(track.track(_("Track %s") % 1, self))

		# init markers
		self.markers = []
		
		# reference to play_head goocanvas group
		self.play_head = None
		self.ruler_time = None
		self.play_head_line = None
		
		
	def AddMarker(self, marker_name, position_on_track):
		
		# loop through existing markers (check for duplicates)
		for exiting_marker in self.markers:
			if exiting_marker.position_on_track == position_on_track:
				return
		
		# Create marker object
		m = marker.marker(marker_name, position_on_track, self)
		
		# Add to Marker Collection
		self.markers.append(m)
		
		# re-order marker collection (to keep them in order of their positions)
		self.reorder_markers()
		
		# mark project as modified
		self.project.set_project_modified(is_modified=True, refresh_xml=True, type=_("Added marker"))
		
		return m

		

	def get_marker(self, direction, current_position):
		

		# check for markers
		if len(self.markers) == 0:
			return None

		if direction == "left":
			
			# loop through all markers (check for duplicates)
			possible_match = None
			for m in self.markers:

				if m.position_on_track < current_position - 0.20:
					possible_match = m

			# return the match
			return possible_match
			
		else:
			# right
			
			# loop through all markers (check for duplicates)
			for m in self.markers:

				if m.position_on_track > current_position + 0.20:
					# return the match
					return m
				
				
		# no markers found
		return None
		
		
		
	def AddTrack(self, track_name, position="above", existing_track=None):
		
		
		# Add another track
		if existing_track:
			index_existing_track = self.tracks.index(existing_track)
			# insert next to an existing track
			if position == "above":
				self.tracks.insert(index_existing_track, track.track(track_name, self))
			else:
				self.tracks.insert(index_existing_track + 1, track.track(track_name, self))
				
		else:
			# insert at top of tracks
			self.tracks.insert(0, track.track(track_name, self))

		# mark project as modified
		self.project.set_project_modified(is_modified=True, refresh_xml=True, type=_("Added Track"))

			
	def rename_track(self, current_track, new_name):
		current_track.name = new_name
		
		# mark project as modified
		self.project.set_project_modified(is_modified=True, refresh_xml=True, type=_("Renamed track"))
		
			
	def get_pixels_per_second(self):
		# calculate the number of pixels per second, based on the scale of the timeline
		return float(self.tick_pixels) / float(self.scale)	   
		
	
	def Render(self):

		# Clear the canvases
		self.project.form.MyCanvas_Left.set_root_item(goocanvas.Group())
		self.project.form.TimelineCanvas_Left.set_root_item(goocanvas.Group())
		self.project.form.MyCanvas.set_root_item(goocanvas.Group())
		self.project.form.TimelineCanvas_Right.set_root_item(goocanvas.Group())
		
		# Render Ruler
		self.RenderRuler()
		
		# Render Markers
		self.RenderMarkers()
		
		# loop through each track
		for MyTrack in self.tracks:
			
			# Render track			
			MyTrack.Render()
			
			
		# loop through each track
		for MyTrack in self.tracks:
			
			# loop through each transition, from the bottom up
			for MyTran in MyTrack.transitions:
				
				# Render track			
				MyTran.Render()

			
			
	def GenerateXML(self, dom, xmlParentNode):
		
		# get frames per second
		fps = self.project.fps()
		
		multitrack = dom.createElement("multitrack")
		xmlParentNode.appendChild(multitrack)
		
		# create fake background track (i.e. black background)
		bg_track = track.track("Background Track", self)
		bg_track.parent = self
		bg_end_time = self.Calculate_Length()
		bg_image = files.OpenShotFile(self.project)
		bg_image.name = os.path.join(self.project.IMAGE_DIR, "black.png")
		bg_image.length = bg_end_time
		bg_image.max_frames = round(bg_end_time * fps)
		
		# set remaining length
		remaining_length = bg_end_time
		position = 0.0
		current_length = 0.0
		
		# Add the background black clip in sections (5 minute (i.e. 300 seconds) sections)
		while remaining_length > 0.0:
			# calculate length of this section
			if remaining_length > 300.0:
				current_length = 300.0
				remaining_length -= 300.0
			else:
				current_length = remaining_length
				remaining_length = 0.0
							
			bg_clip = clip.clip("Background Clip", "Gold", position, 0.0, current_length, bg_track, bg_image)
			bg_clip.distort = True
			bg_clip.fill = True
			bg_track.clips.append(bg_clip)
			
			# calculate position for next section
			position += current_length
		
		# add XML for background track
		bg_track.GenerateXML(dom, multitrack, fps=fps)
		
		# loop through each track, from the bottom up
		for MyTrack in reversed(self.tracks):
			
			# Render track			
			MyTrack.GenerateXML(dom, multitrack, fps=fps)
			

	def Calculate_Length(self):
		""" Determine the length of this sequence """
		longest_clip = 0.1
		fps = self.project.fps()
		
		# loop through each track, from the bottom up
		for MyTrack in self.tracks:
			
			# loop through each clip
			for MyClip in MyTrack.clips:
				
				# calculate end of clip
				end_of_clip = MyClip.position_on_track + MyClip.length()
				
				if end_of_clip > longest_clip:
					longest_clip = end_of_clip
			
		# return longest clip
		return longest_clip + 0.1
	
	#----------------------------------------------------------------------
	def RenderRuler(self):
		"""This adds a track to the canvas with 3 images: a left, middle, and right"""
		
		# get the pixels per second from the parent sequence
		pixels_per_second = self.get_pixels_per_second()
		
		# determine position of playhead
		time = timeline.timeline().get_friendly_time(self.play_head_position * 1000) 
		
		# set the top coordinate of this track to the bottom coordinate of the last track
		x = 10	  
		y_top = 12
		
		# get a reference to the 2 main canvas objects & theme
		theme = self.project.theme
		canvas_timeline = self.project.form.MyCanvas
		canvas_timeline_left = self.project.form.MyCanvas_Left
		canvas_left = self.project.form.TimelineCanvas_Left
		canvas_right = self.project.form.TimelineCanvas_Right
		
 		# Get theme settings
 		theme_settings = self.project.theme_settings.settings
		
		# Add an item to the goocanvas
		root_left = canvas_left.get_root_item ()
		root_right = canvas_right.get_root_item ()

		# Load all 3 images
		imgTrack_Left = gtk.image_new_from_file("%s/openshot/themes/%s/ruler_left.png" % (self.project.form.openshot_path, theme))
		imgTrack_Middle = gtk.image_new_from_file("%s/openshot/themes/%s/ruler_middle.png" % (self.project.form.openshot_path, theme))
		imgTrack_Right = gtk.image_new_from_file("%s/openshot/themes/%s/ruler_right.png" % (self.project.form.openshot_path, theme))	   
		imgTrack_Tick = gtk.image_new_from_file("%s/openshot/themes/%s/ruler_tick.png" % (self.project.form.openshot_path, theme))	   

		# Get Height and Width of Images 
		imgTrack_Left_Height = imgTrack_Left.get_pixbuf().get_height()
		imgTrack_Left_Width = imgTrack_Left.get_pixbuf().get_width()
		imgTrack_Right_Width = imgTrack_Right.get_pixbuf().get_width()		

		# Get Size of Window (to determine how wide the middle image should be streched)
		Size_Of_Middle = int(pixels_per_second * self.length)
		
		# resize the canvas height and width
		canvas_timeline.set_bounds (0, 0, Size_Of_Middle + 185, 5000)
		self.project.form.hscrollbar2.set_range(0, Size_Of_Middle + 200)
		self.project.form.on_hscrollbar2_value_changed(self.project.form.hscrollbar2)
		
		# Get the height of the RULER
		ruler_height = theme_settings["timeline"]["ruler"]["height"]
		left_canvas_width = theme_settings["track"]["track_name_text"]["w"]
		
		# Resize the LEFT RULER
		canvas_timeline_left.set_bounds (0, 0, left_canvas_width, 5000)
		canvas_left.set_bounds (0, 0, left_canvas_width, ruler_height)
		canvas_left.set_size_request (0, ruler_height)
		self.project.form.timelineWindowLeft.set_size_request (left_canvas_width, ruler_height)
		self.project.form.scrolledwindow_Left.set_size_request (left_canvas_width, 0)
		
		# Resize the RIGHT RULER
		canvas_right.set_bounds (0, 0, Size_Of_Middle + 200, ruler_height)
		canvas_right.set_size_request (0, ruler_height)
		self.project.form.timelinewindowRight.set_size_request (0, ruler_height)
		
		# Resize RULER Hbox (which holds both the LEFT and RIGHT windows)
		self.project.form.hbox5.set_size_request (0, ruler_height)

		# Resize Middle pixbuf
		middlePixBuf = imgTrack_Middle.get_pixbuf()
		pixbuf_list = self.split_images(middlePixBuf, imgTrack_Left_Height, Size_Of_Middle)
		
		# Create Group (for the track)
		GroupTrack = goocanvas.Group (parent = root_right)

		# Add Left Image to Group
		image1 = goocanvas.Image (parent = root_left,
								  pixbuf = imgTrack_Left.get_pixbuf(),
								  x = x,
								  y = y_top)

		# Add Middle Image to Group (this can be multiple image tiled together
		pixbuf_x = 0
		for pixbuf in pixbuf_list:
			# get width of this pixbuf
			pixbuf_width = pixbuf.get_width()
			
			image2 = goocanvas.Image (parent = GroupTrack,
									  pixbuf = pixbuf,
									  x = pixbuf_x,
									  y = y_top)  
			
			# increment the x
			pixbuf_x = pixbuf_x + pixbuf_width


		# Add Middle Image to Group
		image3 = goocanvas.Image (parent = GroupTrack,
								  pixbuf = imgTrack_Right.get_pixbuf(),
								  x = Size_Of_Middle - 1,
								  y = y_top)

		# Add Playhead position text
		self.ruler_time = goocanvas.Text (parent = root_left,
								text = theme_settings["timeline"]["playhead_text"]["font"] % (time[2], time[3], time[4], time[5]),
								antialias = True,
								use_markup = True,
								x = x + theme_settings["timeline"]["playhead_text"]["x"],
								y = y_top + theme_settings["timeline"]["playhead_text"]["y"])
			  
		# Resize tick marks for ruler
		tickPixBuf = imgTrack_Tick.get_pixbuf()
		big_tickPixBuf = tickPixBuf.scale_simple(1, theme_settings["timeline"]["ruler"]["large_tick"]["h"], gtk.gdk.INTERP_NEAREST)
		medium_tickPixBuf = tickPixBuf.scale_simple(1, theme_settings["timeline"]["ruler"]["medium_tick"]["h"], gtk.gdk.INTERP_NEAREST)
		small_tickPixBuf = tickPixBuf.scale_simple(1, theme_settings["timeline"]["ruler"]["small_tick"]["h"], gtk.gdk.INTERP_NEAREST)

		# loop through each tick mark
		number_of_ticks = int(self.length / self.scale)
		ruler_pixels = ((float(self.length) / float(self.scale)) * self.tick_pixels)
		tick = 0
		end_of_timeline = False
		
		while end_of_timeline == False:
			
			# increment tick
			tick = tick + 1

			# get the formatted time code
			milliseconds = (tick * self.scale) * 1000
			time = timeline.timeline().get_friendly_time(milliseconds)
			
			if (tick * self.tick_pixels + 16) < ruler_pixels:

				# Add Text to the Track
				text1 = goocanvas.Text (parent = GroupTrack,
										text = theme_settings["timeline"]["ruler"]["time_text"]["font"] % (time[2], time[3], time[4], time[5]),
										antialias = True,
										use_markup = True,
										x = (tick * self.tick_pixels) - theme_settings["timeline"]["ruler"]["time_text"]["x"],
										y = y_top + theme_settings["timeline"]["ruler"]["time_text"]["y"])				

				# Add BIG TICK Image to Group
				image5 = goocanvas.Image (parent = GroupTrack,
										  pixbuf = big_tickPixBuf,
										  x = (tick * self.tick_pixels),
										  y = y_top + theme_settings["timeline"]["ruler"]["large_tick"]["y"])
	
			
			# determine # of minor ticks.  The longer the timeline, the less
			# ticks can be added do to performance reasons.
			number_of_minor_ticks = 6
			if (number_of_ticks * 10) > 1500:
				number_of_minor_ticks = 2
				
			# Add Minor ticks
			big_tick_x = ((tick - 1) * self.tick_pixels)
			minor_tick_increment = self.tick_pixels / number_of_minor_ticks
 
			# loop through minor ticks, and add them to the canvas
			for minor_tick in range(1, number_of_minor_ticks):

				# Resize minor tick pixbuf
				tickPixBuf = None
				minor_y_adjustment = theme_settings["timeline"]["ruler"]["small_tick"]["y"]
				if minor_tick == int(number_of_minor_ticks / 2):
					minor_y_adjustment = theme_settings["timeline"]["ruler"]["medium_tick"]["y"]
					tickPixBuf = medium_tickPixBuf
				else:
					tickPixBuf = small_tickPixBuf
				
				# don't extend past the ruler
				if (big_tick_x + int(minor_tick_increment * minor_tick)) < ruler_pixels:

					# Add minor tick Image to Group
					image5 = goocanvas.Image (parent = GroupTrack,
											  pixbuf = tickPixBuf,
											  x = big_tick_x + int(minor_tick_increment * minor_tick),
											  y = y_top + minor_y_adjustment)
				else:
					end_of_timeline = True
					break

		# Connect signals to ruler to allow drag and drop
		GroupTrack.connect ("button_press_event", self.on_ruler_press)
		GroupTrack.connect ("motion_notify_event", self.on_ruler_motion)
		

				

	def on_ruler_press(self, item, target, event):
		
		if event.button == 1:
			new_x = event.x
			
			# get the pixels per second from the parent sequence
			pixels_per_second = self.get_pixels_per_second()

			# calculate playhead position
			self.play_head_position = new_x / pixels_per_second
			
			if self.play_head_position < 0:
				self.play_head_position = 0
			
			# move playhead
			self.project.form.MyVideo.move_play_head(self.play_head_position)
			
			# update video frame
			self.update_video()

			

	def on_ruler_motion(self, item, target, event):
		#print "on_ruler_motion"

		if (event.state & gtk.gdk.BUTTON1_MASK):
			new_x = event.x
			
			# get the pixels per second from the parent sequence
			pixels_per_second = self.get_pixels_per_second()

			# calculate playhead position
			self.play_head_position = new_x / pixels_per_second
			
			if self.play_head_position < 0:
				self.play_head_position = 0
			
			# move playhead
			self.project.form.MyVideo.move_play_head(self.play_head_position)
			
			# update video frame
			self.update_video()
	
	
	#----------------------------------------------------------------------
	def RenderMarkers(self):
		
		# loop through all markers (check for duplicates)
		for m in self.markers:
			
			# render the marker
			m.Render()
			
				
				
	def reorder_markers(self):
		# get a list of all clips on this track
		self.markers.sort(self.compare_marker)

		
		
	def compare_marker(self, marker1, marker2):
		if marker1.position_on_track > marker2.position_on_track:
			return 1
		elif marker1.position_on_track == marker2.position_on_track:
			return 0
		else:
			return -1

				
	#----------------------------------------------------------------------
	def RenderPlayHead(self):
		"""This adds the playhead to the canvas, and the play head position line"""
		
 		# Get theme settings
 		theme_settings = self.project.theme_settings.settings
		
		# get the pixels per second from the parent sequence
		pixels_per_second = self.get_pixels_per_second()
		
		# set the top coordinate of this track to the bottom coordinate of the last track
		x = self.play_head_position * pixels_per_second
		x = x + theme_settings["timeline"]["ruler"]["playhead"]["x"]
		y_top = theme_settings["timeline"]["ruler"]["playhead"]["y"]
		
		# get a reference to the 2 main canvas objects & theme
		theme = self.project.theme
		canvas_right = self.project.form.TimelineCanvas_Right
		
		# Add an item to the goocanvas
		root_right = canvas_right.get_root_item ()

		# Load all 3 images
		imgTrack_PlayHead = gtk.image_new_from_file("%s/openshot/themes/%s/play_head.png" % (self.project.form.openshot_path, theme))
		imgTrack_Line = gtk.image_new_from_file("%s/openshot/themes/%s/position_line.png" % (self.project.form.openshot_path, theme))
		imgTrack_Ruler = gtk.image_new_from_file("%s/openshot/themes/%s/ruler_right.png" % (self.project.form.openshot_path, theme))
		imgTrack_Track = gtk.image_new_from_file("%s/openshot/themes/%s/Track_Middle.png" % (self.project.form.openshot_path, theme))

		# Get Height and Width of Images 
		imgTrack_PlayHead_Height = imgTrack_PlayHead.get_pixbuf().get_height()
		imgTrack_PlayHead_Width = imgTrack_PlayHead.get_pixbuf().get_width()
		imgTrack_Ruler_Height = imgTrack_Ruler.get_pixbuf().get_height()
		imgTrack_Track_Height = imgTrack_Track.get_pixbuf().get_height()

		# Get Size of Window (to determine how wide the middle image should be streched)
		Size_Of_Line = len(self.tracks) * imgTrack_Track_Height + (len(self.tracks) * theme_settings["track"]["padding"]) + 2

		# Resize Middle pixbuf
		linePixBuf = imgTrack_Line.get_pixbuf()
		linePixBuf = linePixBuf.scale_simple(1, Size_Of_Line, gtk.gdk.INTERP_NEAREST)
		
		# Create Group (for the track)
		GroupTrack = goocanvas.Group (parent = root_right)
		self.play_head = GroupTrack

		# Create the red line
		lineRect = goocanvas.Rect(parent = GroupTrack,
								  x = x,
								  y = y_top + theme_settings["timeline"]["ruler"]["playhead_line"]["y"],
								  width = 1,
								  height = Size_Of_Line,
								  line_width = theme_settings["timeline"]["ruler"]["playhead_line"]["line_width"],
								  stroke_color_rgba = theme_settings["timeline"]["ruler"]["playhead_line"]["stroke_color"]
								  )
		
		# Create Group (for the extended play line)
		track_canvas_right = self.project.form.MyCanvas
		track_root_right = track_canvas_right.get_root_item ()
		GroupTrack2 = goocanvas.Group (parent = track_root_right)
		self.play_head_line = GroupTrack2

		# Create the red line
		lineRect = goocanvas.Rect(parent = GroupTrack2,
								  x = x,
								  y = 0,
								  width = 1,
								  height = Size_Of_Line + theme_settings["timeline"]["ruler"]["playhead_line"]["length_offset"],
								  line_width = theme_settings["timeline"]["ruler"]["playhead_line"]["line_width"],
								  stroke_color_rgba = theme_settings["timeline"]["ruler"]["playhead_line"]["stroke_color"]
								  )

		# Add Play Head Image to Group
		image1 = goocanvas.Image (parent = GroupTrack,
								  pixbuf = imgTrack_PlayHead.get_pixbuf(),
								  x = x + (imgTrack_PlayHead_Width / 2) * -1,
								  y = imgTrack_Ruler_Height - imgTrack_PlayHead_Height + y_top - 2)

		# Connect signals to play head to allow drag and drop
		GroupTrack.connect ("motion_notify_event", self.on_motion_notify_x)
		GroupTrack.connect ("button_press_event", self.on_button_press_x)
		GroupTrack.connect ("button_release_event", self.on_button_release_x)
		
		
		
	def on_motion_notify_x (self, item, target, event):
		"""this method allows the clip to be dragged and dropped on a track"""

		if (event.state & gtk.gdk.BUTTON1_MASK):
			new_x = event.x
			new_y = event.y
			
			# get the pixels per second from the parent sequence
			pixels_per_second = self.get_pixels_per_second()
			
			# get width of playhead
			playhead_width = item.get_bounds().x2 - item.get_bounds().x1
			min_x = (playhead_width / 2) * -1

			# don't allow the clip to slide past the beginning of the canvas
			total_x_diff = new_x - self.drag_x 
			total_y_diff = event.y - self.drag_y
			if (item.get_bounds().x1 + total_x_diff < min_x):
				total_x_diff = min_x - item.get_bounds().x1

			# move clip				
			item.translate (total_x_diff, 0)
			self.play_head_line.translate(total_x_diff, 0)

			# update playhead position
			self.play_head_position = (item.get_bounds().x1 + (playhead_width / 2)) / pixels_per_second
			
			# update video frame
			self.update_video()

		return True
	
	
	def update_video(self):
		
		# return to "preview" mode (if in override mode)
		if self.project.form.MyVideo.mode == "override":
			self.project.set_project_modified(is_modified=True, refresh_xml=True)

		# Refresh the MLT XML file
		self.project.RefreshXML()
		
		# determine new frame number
		frame = self.project.fps() * self.play_head_position
		
		# seek to the new frame
		self.project.form.MyVideo.seek(int(frame))
			

	def move_play_head (self, new_time):
		"""this method allows the play head to be moved to a specific spot on the timeline.  It accepts
		a parameter for the # of seconds to move the playhead to."""
		
		# move play_head to the top layer
		if self.play_head and new_time != self.play_head_position:

			self.play_head.raise_(None)
			self.play_head_line.raise_(None)
			
			# get the pixels per second from the parent sequence
			pixels_per_second = self.get_pixels_per_second()
			
			# get width of playhead
			playhead_width = self.play_head.get_bounds().x2 - self.play_head.get_bounds().x1
			min_x = (playhead_width / 2) * -1
	
			# don't allow the clip to slide past the beginning of the canvas
			total_x_diff = (new_time * pixels_per_second) - (self.play_head.get_bounds().x2 + min_x)
	
			# move clip				
			self.play_head.translate (total_x_diff, 0)
			self.play_head_line.translate (total_x_diff, 0)
			
	 		# Get theme settings
	 		theme_settings = self.project.theme_settings.settings
			
			# print time of playhead
			milliseconds = ((self.play_head.get_bounds().x1 + (playhead_width / 2)) / pixels_per_second) * 1000
			time = timeline.timeline().get_friendly_time(milliseconds)
			frame = round((time[5] / 1000.0) * self.project.fps())
			self.ruler_time.set_property("text", theme_settings["timeline"]["playhead_text"]["font"] % (time[2], time[3], time[4], frame))
						
			# update playhead position
			self.play_head_position = (self.play_head.get_bounds().x1 + (playhead_width / 2)) / pixels_per_second

	
	
	def on_button_press_x (self, item, target, event):
		""" This method initializes some variables needed for dragging and dropping a clip """

		# raise the group up to the top level
		item.raise_(None)

		if event.button == 1:

			# set the x and y where the cursor started dragging from
			self.drag_x = event.x
			self.drag_y = event.y

			# change the cursor for the drag n drop operation
			fleur = gtk.gdk.Cursor (gtk.gdk.FLEUR)
			canvas = item.get_canvas ()
			canvas.pointer_grab (item,
								 gtk.gdk.POINTER_MOTION_MASK | gtk.gdk.BUTTON_RELEASE_MASK,
								 fleur,
								 event.time)
			
			# pause the video
			self.project.form.MyVideo.pause()
			
		elif event.button == 3:
			# show right click menu
			self.project.form.mnuPlayheadSubMenu1.showmnu(event, self, item)

		return True
	
	
	def on_button_release_x (self, item, target, event):
		
		""" This method drops a clip, and snaps the clip to the nearest valid track """
		# get reference to the canvas, and stop dragging the item
		canvas = item.get_canvas ()
		canvas.pointer_ungrab (item, event.time)
	
				
		
	def split_images(self, pixbuf, height, max_length):
		""" Because it's not possible to resize an image to an infinate size, we sometimes
		need to split an image into many smaller pieces.  This function takes an image, and
		returns a list of pixbufs to equal the max_length"""
		
		# how big of chunks do we use (pixels)
		chunk_size = 1000
		image_list = []
		
		# get a reference to the 2 main canvas objects & theme
		remaining_pixels = max_length
		
		while remaining_pixels:
			# declare new image
			new_pixbuf = None

			if chunk_size < remaining_pixels:
				# resize the pixbuf
				new_pixbuf = pixbuf.scale_simple(chunk_size, height, gtk.gdk.INTERP_NEAREST)
				
				# subtract the pixels
				remaining_pixels = remaining_pixels - chunk_size
				
			else:
				# resize the pixbuf
				new_pixbuf = pixbuf.scale_simple(remaining_pixels, height, gtk.gdk.INTERP_NEAREST)

				# subtract the pixels
				remaining_pixels = 0
				
			# add image to list
			image_list.append(new_pixbuf)
		
		# return list of pixbufs
		return image_list
	
	
	def resize_image_list(self, image_list, height, new_length):
		""" Because it's not possible to resize an image to an infinate size, we sometimes
		need to split an image into many smaller pieces.  This function takes an image, and
		returns a list of pixbufs to equal the max_length"""

		# create new list
		new_image_list = []

		# get a reference to the 2 main canvas objects & theme
		remaining_pixels = new_length
		

		# loop through existing pixbufs
		for current_pixbuf in image_list:
			
			# get width of pixbuf
			pixbuf_width = current_pixbuf.get_width()
			
			if remaining_pixels == 0:
				pass
			
			# Is Last Image?
			elif pixbuf_width >= remaining_pixels:

				# resize the pixbuf
				new_pixbuf = current_pixbuf.scale_simple(remaining_pixels, height, gtk.gdk.INTERP_NEAREST)
				new_image_list.append(new_pixbuf)
				remaining_pixels = 0
				
			else:
				# not the last image
				# subtract the pixels
				new_image_list.append(current_pixbuf)
				remaining_pixels = remaining_pixels - pixbuf_width
				
				
		# check for additional pixbufs needed?
		if remaining_pixels > 0:
			# resize the pixbuf
			new_pixbuf = image_list[0].scale_simple(remaining_pixels, height, gtk.gdk.INTERP_NEAREST)
			
			# add to image list
			new_image_list.append(new_pixbuf)
		
		
		# return list of pixbufs
		return new_image_list
	

		
	def raise_transitions(self):
		"""this method loops though the children objects of this group looking 
		for the item with a specfic id."""

		# Get root group of the canvas
		canvas_right = self.project.form.MyCanvas
		root_right = canvas_right.get_root_item ()	
		
		# Build list of canvas IDs
		IDs = {}
		for index in range(0, root_right.get_n_children()):
			child = root_right.get_child(index)
			child_id = child.get_data ("id")
			
			# add to list
			IDs[str(child_id)] = child
		
		
		# loop through each transtion
		for track in self.tracks:
			for trans in track.transitions:
			
				# see if there is a matching transition
				if trans.unique_id in IDs:
					# raise match to the top layer
					canvas_item = IDs[trans.unique_id]
					canvas_item.raise_(None)
			
	def raise_play_head(self):
		""" Raise the play-head to the top """
		self.play_head.raise_(None)
		self.play_head_line.raise_(None)

	def get_valid_track(self, x1, y1):
		""" A clip must be dropped on a track.  This method returns the track 
		object that is under the clip's current position """

		# loop through each track
		for track in self.tracks:
			# get the top y and bottom y of each track
			y_top = track.y_top
			y_bottom = track.y_bottom

			# get the middle of the clip
			half_height_of_clip = 0
			middle_position = half_height_of_clip + y1
			
			# determine if middle of clip is contained inside this track
			if middle_position > y_top and middle_position < y_bottom:
				return track
		
		# return false if no valid track found
		return None
		
		
		
		



#	OpenShot Video Editor is a program that creates, modifies, and edits video files.
#   Copyright (C) 2009  Jonathan Thomas
#
#	This file is part of OpenShot Video Editor (http://launchpad.net/openshot/).
#
#	OpenShot Video Editor is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	OpenShot Video Editor is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with OpenShot Video Editor.  If not, see <http://www.gnu.org/licenses/>.

import os, sys, subprocess, locale, gettext, datetime
import xml.dom.minidom as xml
import openshot.classes.info as info

# This file helps you generate the POT file that contains all of the translatable
# strings / text in OpenShot.  Because some of our text is in custom XML files,
# the xgettext command can't correctly generate the POT file.  Thus... the 
# existance of this file. =)

# Command to create the individual language PO files (Ascii files)
#		$ msginit --input=OpenShot.pot --locale=fr_FR
#		$ msginit --input=OpenShot.pot --locale=es

# Command to update the PO files (if text is added or changed)
#		$ msgmerge en_US.po OpenShot.pot -U
#		$ msgmerge es.po OpenShot.pot -U

# Command to compile the Ascii PO files into binary MO files
#		$ msgfmt en_US.po --output-file=en_US/LC_MESSAGES/OpenShot.mo
#		$ msgfmt es.po --output-file=es/LC_MESSAGES/OpenShot.mo

# Command to compile all PO files in a folder
#		$ find -iname "*.po" -exec msgfmt {} -o {}.mo \;

# Command to combine the 2 pot files into 1 file
#       $ msgcat ~/openshot/locale/OpenShot/OpenShot_source.pot ~/openshot/openshot/locale/OpenShot/OpenShot_glade.pot -o ~/openshot/main/locale/OpenShot/OpenShot.pot


# get the path of the main OpenShot folder
langage_folder_path = os.path.dirname(os.path.abspath(__file__))
openshot_path = os.path.dirname(langage_folder_path)
effects_path = os.path.join(openshot_path, 'effects')
blender_path = os.path.join(openshot_path, 'blender')
transitions_path = os.path.join(openshot_path, 'transitions')
export_path = os.path.join(openshot_path, 'export_presets')
locale_path = os.path.join(openshot_path, 'locale', 'OpenShot')

print "-----------------------------------------------------"
print " Creating 4 temp POT files"
print "-----------------------------------------------------"

# create empty temp files in the /openshot/language folder (these are used as temp POT files)
temp_files = ['OpenShot_source', 'OpenShot_glade', 'OpenShot_effects', 'OpenShot_export', 'OpenShot_transitions']
for temp_file in temp_files:
	if os.path.exists(temp_file):
		os.remove(temp_file)
	f = open(temp_file, "w")
	f.close()
	
print "-----------------------------------------------------"
print " Using xgettext to generate .Glade and .py POT files"
print "-----------------------------------------------------"

# Generate POT for Source Code strings (i.e. strings marked with a _("translate me"))
subprocess.call('find %s -iname "*.py" -exec xgettext -j -o %s --keyword=_ {} \;' % (openshot_path, os.path.join(langage_folder_path, 'OpenShot_source')), shell=True)
subprocess.call('find %s -iname "*.ui" -exec xgettext -j -L Glade -o %s --keyword=translatable {} \;' % (openshot_path, os.path.join(langage_folder_path, 'OpenShot_glade')), shell=True)

print "-----------------------------------------------------"
print " Updating auto created POT files to set CharSet"
print "-----------------------------------------------------"

temp_files = ['OpenShot_source', 'OpenShot_glade']
for temp_file in temp_files:
	# get the entire text
	f = open(os.path.join(langage_folder_path, temp_file), "r")
	# read entire text of file
	entire_source = f.read()
	f.close()
	
	# replace charset
	entire_source = entire_source.replace("charset=CHARSET", "charset=UTF-8")
	
	# Create Updated POT Output File
	if os.path.exists(os.path.join(langage_folder_path, temp_file)):
		os.remove(os.path.join(langage_folder_path, temp_file))
	f = open(os.path.join(langage_folder_path, temp_file), "w")
	f.write(entire_source)
	f.close()
	

print "-----------------------------------------------------"
print " Scanning custom XML files and finding text"
print "-----------------------------------------------------"

# Loop through the Effects XML
effects_text = {}
for file in os.listdir(effects_path):
	if os.path.isfile(os.path.join(effects_path, file)):
		# load xml effect file
		full_file_path = os.path.join(effects_path, file)
		xmldoc = xml.parse(os.path.join(effects_path, file))

		# add text to list
		effects_text[xmldoc.getElementsByTagName("title")[0].childNodes[0].data] = full_file_path
		effects_text[xmldoc.getElementsByTagName("description")[0].childNodes[0].data] = full_file_path
		
		# get params
		params = xmldoc.getElementsByTagName("param")
		
		# Loop through params
		for param in params:
			if param.attributes["title"]:
				effects_text[param.attributes["title"].value] = full_file_path
				
				
# Loop through the Blender XML
for file in os.listdir(blender_path):
	if os.path.isfile(os.path.join(blender_path, file)):
		# load xml effect file
		full_file_path = os.path.join(blender_path, file)
		xmldoc = xml.parse(os.path.join(blender_path, file))

		# add text to list
		effects_text[xmldoc.getElementsByTagName("title")[0].childNodes[0].data] = full_file_path
		#effects_text[xmldoc.getElementsByTagName("description")[0].childNodes[0].data] = full_file_path
		
		# get params
		params = xmldoc.getElementsByTagName("param")
		
		# Loop through params
		for param in params:
			if param.attributes["title"]:
				effects_text[param.attributes["title"].value] = full_file_path


# Loop through the Export Settings XML
export_text = {}
for file in os.listdir(export_path):
	if os.path.isfile(os.path.join(export_path, file)):
		# load xml export file
		full_file_path = os.path.join(export_path, file)
		xmldoc = xml.parse(os.path.join(export_path, file))

		# add text to list
		export_text[xmldoc.getElementsByTagName("type")[0].childNodes[0].data] = full_file_path
		export_text[xmldoc.getElementsByTagName("title")[0].childNodes[0].data] = full_file_path

# Loop through transitions and add to POT file
transitions_text = {}
for file in os.listdir(transitions_path):
	# load xml export file
	full_file_path = os.path.join(transitions_path, file)
	(fileBaseName, fileExtension)=os.path.splitext(file)
	
	# get transition name
	name = fileBaseName.replace("_", " ").capitalize()
	
	# add text to list
	transitions_text[name] = full_file_path



print "-----------------------------------------------------"
print " Creating the custom XML POT files"
print "-----------------------------------------------------"

# header of POT file
header_text = ""
header_text = header_text + '# OpenShot Video Editor POT Template File.\n'
header_text = header_text + '# Copyright (C) 2009  Jonathan Thomas\n'
header_text = header_text + '# This file is distributed under the same license as the PACKAGE package.\n'
header_text = header_text + '# Jonathan Thomas <Jonathan.Oomph@gmail.com>, 2009.\n'
header_text = header_text + '#\n'
header_text = header_text + '#, fuzzy\n'
header_text = header_text + 'msgid ""\n'
header_text = header_text + 'msgstr ""\n'
header_text = header_text + '"Project-Id-Version: OpenShot Video Editor (version: %s)\\n"\n' % info.VERSION
header_text = header_text + '"Report-Msgid-Bugs-To: Jonathan Thomas <Jonathan.Oomph@gmail.com>\\n"\n'
header_text = header_text + '"POT-Creation-Date: %s\\n"\n' % datetime.datetime.now()
header_text = header_text + '"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"\n'
header_text = header_text + '"Last-Translator: Jonathan Thomas <Jonathan.Oomph@gmail.com>\\n"\n'
header_text = header_text + '"Language-Team: https://translations.launchpad.net/+groups/launchpad-translators\\n"\n'
header_text = header_text + '"MIME-Version: 1.0\\n"\n'
header_text = header_text + '"Content-Type: text/plain; charset=UTF-8\\n"\n'
header_text = header_text + '"Content-Transfer-Encoding: 8bit\\n"\n'

# Create POT files for the custom text (from our XML files)
temp_files = [['OpenShot_effects',effects_text], ['OpenShot_export',export_text], ['OpenShot_transitions',transitions_text]]
for temp_file, text_dict in temp_files:
	f = open(temp_file, "w")
	
	# write header
	f.write(header_text)
	
	# loop through each line of text
	for k,v in text_dict.items():
		if k:
			f.write('\n')
			f.write('#: %s\n' % v)
			f.write('msgid "%s"\n' % k)
			f.write('msgstr ""\n')
	
	# close file
	f.close()
	
	
print "-----------------------------------------------------"
print " Combine all 4 temp POT files using msgcat command (this removes dupes) "
print "-----------------------------------------------------"
	
temp_files = ['OpenShot_source', 'OpenShot_glade', 'OpenShot_effects', 'OpenShot_export', 'OpenShot_transitions']
command = "msgcat"
for temp_file in temp_files:
	# append files
	command = command + " " + os.path.join(langage_folder_path, temp_file)
command = command + " -o " + os.path.join(locale_path, "OpenShot.pot")

print command

# merge all 4 temp POT files
subprocess.call(command, shell=True)
	
	
print "-----------------------------------------------------"
print " Create FINAL POT File from 4 temp POT files "
print "-----------------------------------------------------"

# get the entire text of OpenShot.POT
f = open(os.path.join(locale_path, "OpenShot.pot"), "r")
# read entire text of file
entire_source = f.read()
f.close()

# Create Final POT Output File
if os.path.exists(os.path.join(locale_path, "OpenShot.pot")):
	os.remove(os.path.join(locale_path, "OpenShot.pot"))
final = open(os.path.join(locale_path, "OpenShot.pot"), "w")
final.write(header_text)
final.write("\n")

# Trim the beginning off of each POT file
start_pos = entire_source.find("#: ")
trimmed_source = entire_source[start_pos:]

# Add to Final POT File
final.write(trimmed_source)
final.write("\n")
	
# Close final POT file
final.close()


print "-----------------------------------------------------"
print " Remove 4 temp POT files "
print "-----------------------------------------------------"

# Delete all 4 temp files
temp_files = ['OpenShot_source', 'OpenShot_glade', 'OpenShot_effects', 'OpenShot_export', 'OpenShot_transitions']
for temp_file in temp_files:
	if os.path.exists(temp_file):
		os.remove(temp_file)
		
# output success
print "-----------------------------------------------------"
print " The /openshot/locale/OpenShot/OpenShot.pot file has"
print " been successfully created with all text in OpenShot."
print "-----------------------------------------------------"


